from argparse import ArgumentParser

from .constants import NugitAbout

parser = ArgumentParser(
    prog=f"{NugitAbout.NAME}",
    description=NugitAbout.DESCRIPTION,
    add_help=True
)

parser.add_argument(
    "action",
    choices=("mount", "validate", "run", "remove"),
    help="specific action to perform",
)



from ..actions import mount, remove, run
from ..argsparser import parser


def nugit():
    args = parser.parse_args()
    match args.action:
        case "mount":
            mount()
        case "validate":
            raise NotImplementedError
        case "run":
            run()
        case "remove":
            remove()



import os.path
import shutil
import stat
import subprocess

from .constants import NugitPaths


def mount():
    if not os.path.exists(NugitPaths.CFG_DEST):
        shutil.copy2(NugitPaths.DEFAULT_CFG_FILEPATH, os.path.curdir)

    shutil.copy2(NugitPaths.DEFAULT_PRECOMMIT_FILEPATH, NugitPaths.PRECOMMIT_DEST)

    st = os.stat(NugitPaths.PRECOMMIT_DEST)
    os.chmod(NugitPaths.PRECOMMIT_DEST, st.st_mode | stat.S_IEXEC)


def run():
    if os.path.exists(NugitPaths.PRECOMMIT_DEST):
        p = subprocess.Popen([NugitPaths.PRECOMMIT_DEST])
        p.communicate()


def remove():
    if os.path.exists(NugitPaths.CFG_DEST):
        os.remove(NugitPaths.CFG_DEST)

    if os.path.exists(NugitPaths.PRECOMMIT_DEST):
        os.remove(NugitPaths.PRECOMMIT_DEST)



import os
from dataclasses import dataclass


@dataclass(init=False, frozen=True)
class NugitAbout:
    NAME: str = "nugit"
    DESCRIPTION: str = "Easy to understand, configure and install git pre-commit solution"
    VERSION: str = "1.0.6"
    AUTHOR: str = "Lev Zlobin (lengthylyova)"
    AUTHOR_EMAIL: str = "lengthylyova@gmail.com"


@dataclass(init=False, frozen=True)
class NugitPaths:
    DEFAULT_CFG_FILEPATH: str = os.path.join(os.path.dirname(__file__), "defaults/nugit.yaml")
    DEFAULT_PRECOMMIT_FILEPATH: str = os.path.join(os.path.dirname(__file__), "defaults/pre-commit")

    CFG_DEST: str = os.path.join(os.path.curdir, "nugit.yaml")
    PRECOMMIT_DEST: str = os.path.join(os.path.curdir, ".git/hooks/pre-commit")
